@@ -23,6 +23,16 @@
  */
 var g_collapse_clear = 1;
 
+/**
+ * MantisBT config options.
+ * Initialized in javascript_config.php
+ * @property {string} config.collapse_settings_cookie
+ * @property {string} config.cookie_path
+ * @property {string} config.cookie_domain
+ * @property {string} config.cookie_samesite
+ */
+/* global config */
+
 // global code to determine how to set visibility
 var a = navigator.userAgent.indexOf("MSIE");
 var style_display;
@@ -35,18 +45,18 @@ if (a!= -1) {
 style_display = 'block';
 
 $(document).ready( function() {
-    $('.collapse-open').show();
-    $('.collapse-closed').hide();
-    $('.collapse-link').click( function(event) {
-        event.preventDefault();
-        var id = $(this).attr('id');
-        var t_pos = id.indexOf('_closed_link' );
-        if( t_pos == -1 ) {
-            t_pos = id.indexOf('_open_link' );
-        }
-        var t_div = id.substring(0, t_pos );
-        ToggleDiv( t_div );
-    });
+	$('.collapse-open').show();
+	$('.collapse-closed').hide();
+	$('.collapse-link').click( function(event) {
+		event.preventDefault();
+		var id = $(this).attr('id');
+		var t_pos = id.indexOf('_closed_link' );
+		if( t_pos == -1 ) {
+			t_pos = id.indexOf('_open_link' );
+		}
+		var t_div = id.substring(0, t_pos );
+		ToggleDiv( t_div );
+	});
 
 	/**
 	 * Manage the navbar project menu initializacion and events
@@ -121,31 +131,31 @@ $(document).ready( function() {
 		$('#projects-list .search').focus();
 	});
 
-    $('.widget-box').on('shown.ace.widget' , function(event) {
-       var t_id = $(this).attr('id');
-       var t_cookie = GetCookie( "collapse_settings" );
-        if ( 1 == g_collapse_clear ) {
-            t_cookie = "";
-            g_collapse_clear = 0;
-        }
-        t_cookie = t_cookie.replace("|" + t_id + ":1", '' );
-        t_cookie = t_cookie + "|" + t_id + ":0";
-        SetCookie( "collapse_settings", t_cookie );
+	$('.widget-box').on('shown.ace.widget' , function(event) {
+		var t_id = $(this).attr('id');
+		var t_cookie = GetCookie( config.collapse_settings_cookie );
+		if ( 1 == g_collapse_clear ) {
+			t_cookie = "";
+			g_collapse_clear = 0;
+		}
+		t_cookie = t_cookie.replace("|" + t_id + ":1", '' );
+		t_cookie = t_cookie + "|" + t_id + ":0";
+		SetCookie( config.collapse_settings_cookie, t_cookie );
 	});
 
-    $('.widget-box').on('hidden.ace.widget' , function(event) {
-        var t_id = $(this).attr('id');
-        var t_cookie = GetCookie( "collapse_settings" );
-        if ( 1 == g_collapse_clear ) {
-            t_cookie = "";
-            g_collapse_clear = 0;
-        }
-        t_cookie = t_cookie.replace( "|" + t_id + ":0", '' );
-        t_cookie = t_cookie + "|" + t_id + ":1";
-        SetCookie( "collapse_settings", t_cookie );
-    });
+	$('.widget-box').on('hidden.ace.widget' , function(event) {
+		var t_id = $(this).attr('id');
+		var t_cookie = GetCookie( config.collapse_settings_cookie );
+		if ( 1 == g_collapse_clear ) {
+			t_cookie = "";
+			g_collapse_clear = 0;
+		}
+		t_cookie = t_cookie.replace( "|" + t_id + ":0", '' );
+		t_cookie = t_cookie + "|" + t_id + ":1";
+		SetCookie( config.collapse_settings_cookie, t_cookie );
+	});
 
-    $('#sidebar-btn.sidebar-toggle').on('click', function (event) {
+	$('#sidebar-btn.sidebar-toggle').on('click', function (event) {
 		var t_cookie;
 		var t_sidebar = $(this).closest('.sidebar');
 		var t_id = t_sidebar.attr('id');
@@ -155,7 +165,7 @@ $(document).ready( function() {
 			g_collapse_clear = 0;
 		} else {
 			// Get collapse state and remove the old value
-			t_cookie = GetCookie("collapse_settings");
+			t_cookie = GetCookie(config.collapse_settings_cookie);
 			t_cookie = t_cookie.replace(new RegExp('\\|' + t_id + ':.'), '');
 		}
 
@@ -163,11 +173,11 @@ $(document).ready( function() {
 		var t_value = !t_sidebar.hasClass("menu-min") | 0;
 		t_cookie += '|' + t_id + ':' + t_value;
 
-		SetCookie("collapse_settings", t_cookie);
-    });
+		SetCookie(config.collapse_settings_cookie, t_cookie);
+	});
 
-    $('input[type=text].typeahead').each(function() {
-        var $this = $(this);
+	$('input[type=text].typeahead').each(function() {
+		var $this = $(this);
 		$(this).typeahead({
 			minLength: 1,
 			highlight: true
@@ -181,7 +191,7 @@ $(document).ready( function() {
 					$.each(data, function (i, value) {
 						results.push(value);
 					});
-	 				callback(results);
+					callback(results);
 				});
 			}
 		});
@@ -671,7 +681,6 @@ function Trim( p_string ) {
  * Cookie functions
  */
 function GetCookie( p_cookie ) {
-	var t_cookie_name = "MANTIS_" + p_cookie;
 	var t_cookies = document.cookie;
 
 	t_cookies = t_cookies.split( ";" );
@@ -681,8 +690,7 @@ function GetCookie( p_cookie ) {
 		var t_cookie = t_cookies[ i ];
 
 		t_cookie = t_cookie.split( "=" );
-
-		if ( Trim( t_cookie[ 0 ] ) == t_cookie_name ) {
+		if ( Trim( t_cookie[ 0 ] ) == p_cookie ) {
 			return( t_cookie[ 1 ] );
 		}
 		i++;
@@ -692,19 +700,22 @@ function GetCookie( p_cookie ) {
 }
 
 function SetCookie( p_cookie, p_value ) {
-	var t_cookie_name = "MANTIS_" + p_cookie;
 	var t_expires = new Date();
 
 	t_expires.setTime( t_expires.getTime() + (365 * 24 * 60 * 60 * 1000));
 
-	document.cookie = t_cookie_name + "=" + p_value + "; expires=" + t_expires.toUTCString() + ";";
+	document.cookie = p_cookie + "=" + p_value +
+		"; expires=" + t_expires.toUTCString() +
+		( config.cookie_domain ? "; domain=" + config.cookie_domain : '' ) +
+		"; path=" + config.cookie_path +
+		"; samesite=" + config.cookie_samesite;
 }
 
 function ToggleDiv( p_div ) {
 	var t_open_div = '#' + p_div + "_open";
 	var t_closed_div = '#' + p_div + "_closed";
 
-	var t_cookie = GetCookie( "collapse_settings" );
+	var t_cookie = GetCookie( config.collapse_settings_cookie );
 	if ( 1 == g_collapse_clear ) {
 		t_cookie = "";
 		g_collapse_clear = 0;
@@ -717,14 +728,14 @@ function ToggleDiv( p_div ) {
 	}
 
 	if ( t_open_display == "none" ) {
-        t_cookie = t_cookie.replace( "|" + p_div + ":0", '' );
+		t_cookie = t_cookie.replace( "|" + p_div + ":0", '' );
 		t_cookie = t_cookie + "|" + p_div + ":1";
 	} else {
-        t_cookie = t_cookie.replace( "|" + p_div + ":1", '' );
+		t_cookie = t_cookie.replace( "|" + p_div + ":1", '' );
 		t_cookie = t_cookie + "|" + p_div + ":0";
 	}
 
-	SetCookie( "collapse_settings", t_cookie );
+	SetCookie( config.collapse_settings_cookie, t_cookie );
 }
 
 function setDisplay(idTag, state)
@@ -750,7 +761,8 @@ function enableDropzone( classPrefix, autoUpload ) {
 	var zone = $( zone_class );
 	var form = zone.closest('form');
 	var max_filesize_bytes = zone.data('max-filesize-bytes');
-	var max_filseize_mb = Math.ceil( max_filesize_bytes / ( 1024*1024) );
+	var max_filesize_mb = Math.ceil( max_filesize_bytes / ( 1024*1024) );
+	var max_filename_length = zone.data( 'max-filename-length' );
 	var options = {
 		forceFallback: zone.data('force-fallback'),
 		paramName: "ufile",
@@ -759,7 +771,7 @@ function enableDropzone( classPrefix, autoUpload ) {
 		previewsContainer: '#' + classPrefix + '-previews-box',
 		uploadMultiple: true,
 		parallelUploads: 100,
-		maxFilesize: max_filseize_mb,
+		maxFilesize: max_filesize_mb,
 		timeout: 0,
 		addRemoveLinks: !autoUpload,
 		acceptedFiles: zone.data('accepted-files'),
@@ -798,24 +810,40 @@ function enableDropzone( classPrefix, autoUpload ) {
 			 * an array with the added files.
 			 */
 			this.on("addedfiles", function (files) {
-				var error_found = false;
-				var text_files = '';
+				var bullet = '-\u00A0';
+				var error_file_too_big = '';
+				var error_filename_too_long = '';
 				for (var i = 0; i < files.length; i++) {
 					if( files[i].size > max_filesize_bytes ) {
-						error_found = true;
 						var size_mb = files[i].size / ( 1024*1024 );
 						var dec = size_mb < 0.01 ? 3 : 2;
-						text_files = text_files + '"' + files[i].name + '" (' + size_mb.toFixed(dec) + ' MiB)\n';
+						error_file_too_big += bullet + '"' + files[i].name + '" (' + size_mb.toFixed(dec) + ' MiB)\n';
 						this.removeFile( files[i] );
+					} else if( files[i].name.length > 250 ) {
+						error_filename_too_long += bullet + '"' + files[i].name + '" (' + files[i].name.length + ')\n';
+						// this.removeFile( files[i] );
 					}
 				}
-				if( error_found ) {
+
+				var text = '';
+				var error_message = '';
+				if( error_file_too_big ) {
 					var max_mb = max_filesize_bytes / ( 1024*1024 );
 					var max_mb_dec = max_mb < 0.01 ? 3 : 2;
-					var text = zone.data( 'dropzone_multiple_files_too_big' );
-					text = text.replace( '{{files}}', '\n' + text_files + '\n' );
+					text = zone.data( 'dropzone_multiple_files_too_big' ) + "\n";
+					text = text.replace( '{{files}}', '\n' + error_file_too_big );
 					text = text.replace( '{{maxFilesize}}', max_mb.toFixed(max_mb_dec) );
-					alert( text );
+					error_message += text;
+				}
+
+				if( error_filename_too_long ) {
+					text = zone.data( 'dropzone_multiple_filenames_too_long' ) + "\n";
+					text = text.replace( '{{maxFilenameLength}}', max_filename_length );
+					text = text.replace( '{{files}}', '\n' + error_filename_too_long );
+					error_message += text;
+				}
+				if( error_message ) {
+					alert(error_message);
 				}
 			});
 		},
